# $Id: Subscriptions.pm,v 1.2 2004/06/20 15:49:29 rich Exp $

package Subscripto::Subscriptions;

use Carp;
use Email::Folder;
use Email::List;
use Email::List::Subscription;

sub new
{
    my $class = shift(@_);
    my $config = shift(@_);
    my $self = bless({}, $class);

    $self->{config} = $config;

    $self->_clear();

    if (!$self->load())
    {
	carp 'Unable to parse subscriptions';
    }

    return $self;
}

sub _clear
{
    my $self = shift(@_);

    $self->{lists} = {};
    $self->{summaries} = [];

    return 1;
}

# Parse the mails to get the subscriptions. Also produce summaries
# of the subscriptions for use in the GUI.
sub load
{
    my $self = shift(@_);

    # Hash -> hash containing list name, subscription address,
    # subscription status.
    # TODO: Should put in a class.
    # my %lists;

    foreach ($self->{config}->get_folders())
    {
	my $folder = Email::Folder->new($_);
	my $message;

	while ($message = $folder->next_message())
	{
	    # Create the list entry, if any.
	    my @subscriptions = Email::List::Subscription->match($message);
	    my $subscription;

	    foreach $subscription (@subscriptions)
	    {
		my $listname = $subscription->{listname};
		my $list;

		if (!defined($self->{lists}->{$listname}))
		{
		    # New list.
		    $self->{lists}->{$listname}
		      = $list = Email::List->new($listname);
		}
		else
		{
		    $list = $self->{lists}->{$listname};
		}

		$list->add_subscription($subscription);
	    }
	}
    }

    my @listnames = keys(%{$self->{lists}});
    my $listname;

    foreach $listname (@listnames)
    {
	my $subscription;

	# Remove any subscriptions that were later cancelled (unsubscribed).
	$self->{lists}->{$listname}->prune_subscriptions();

	foreach $subscription (@{$self->{lists}->{$listname}->{subscriptions}})
	{
	    my $subscribed
		=    $subscription->{type}
	          == $Email::List::Subscription::SUBSCRIPTION;

	    push(@{$self->{summaries}}, [ $listname,
					  $subscription->{subscriber},
					  $subscribed ]);
	}
    }

    return 1;
}

sub reload
{
    my $self = shift(@_);

    $self->_clear();
    return $self->load();
}

sub lists
{
    my $self = shift(@_);

    return $self->{lists};
}

sub summaries
{
    my $self = shift(@_);

    return $self->{summaries};
}

1;
